# stream2app - cliente em Flutter

:checkered_flag: [English (Inglês)](README.md)

Framework para criar servidores para aplicativos em Flutter que possibilita:

- :handshake: realizar uma conexão websocket para troca de dados que: 
 
    - :vibration_mode: suporta integração com streams no cliente em Flutter
  
    - :arrow_right_hook: realiza o reenvio de dados em caso de instabilidade
    da conexão do cliente

- :pencil2: criar as próprias operações CRUD (**C**reate, **R**ead, **U**pdate e **D**elete)

- :no_entry: restringir acesso do cliente com as operações CRUD

- :mega: notificar em tempo real clientes que estão ouvindo por mudanças de uma rota, podendo ser:
    
    - :no_pedestrians: apenas clientes específicos irão receber os dados
    
    - :heavy_check_mark: todos os clientes irão receber os dados

- :lock: aceitar e recusar tentativas de conexão

Este é lado cliente (Flutter), 
[clique aqui](https://github.com/DesiredApp/stream2app_nodejs_server/blob/master/README_PORTUGUES.md) para
o lado servidor (Node.js).

## Material para referência
*  [Documentação do cliente](documentation/portugues_documentacao.md): Referente ao cliente em Flutter.
*  [chat (exemplo)](example/chat): Troca de mensagens instantâneas entre as cores azul e verde.
*  [catalog (exemplo)](example/catalog): Simulação de múltiplos usuários alterando e removendo produtos de um catálogo.
*  [Começando (servidor)](https://github.com/DesiredApp/stream2app_nodejs_server/blob/master/README_PORTUGUES.md): Referente ao lado servidor.


## Começando

![Alt Text](https://media.giphy.com/media/lT9iLpfjnxiRTVOXe2/giphy.gif)

A seção "Começando" é uma seção exemplo do cliente em Flutter, 
é executado um exemplo localmente.

1 - Primeiramente  [clique aqui](https://github.com/DesiredApp/stream2app_nodejs_server/blob/master/README_PORTUGUES.md) e 
siga as instruções do servidor na seção "Começando"

2 - Realize a instalação

pubspec.yaml:

    dependencies:
      flutter:
        sdk: flutter
        
      # Add this line:
      stream2app: ^0.1.0

3 -  Importe

    import 'package:stream2app/stream2app.dart';

4 - Inicialize o servidor com `Stream2App.instance.init(serverUrl:"ws://<IPV4>:<PORT>")` informando o endereço IPV4 obtido e a porta (padrão: 3123).

5 - Realize a conexão com o servidor com `Stream2App.instance.connect()`
    
Exemplo:

    void main() {
      Stream2App.instance.init(serverUrl:"ws://192.168.2.1:3123");
      Stream2App.instance.connect();
      runApp(TrackingApp());
    }    


6 - Copie e cole esse template dentro de `build`

    @override
    Widget build(BuildContext context) {
      return Scaffold(
          appBar: AppBar(
            title: Text("Tracking"),
          ),
          body: Center(
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                //Show the status of the tracking in realtime
                // FIRST TODO
    
                SizedBox(height: 100,),
    
                //Send data to server with body as text: "I'm waiting"
                // SECOND TODO
               ],
            ),
          )
      );
    }

7 - Substitua `FIRST TODO` por um widget que realiza a leitura em tempo real
 de uma rota do servidor
 
    Stream2App.instance
        .listenAndBuild(
            route: 'product/tracking',
            builder: (context,  snapshot) {
                if(!snapshot.hasData)
                    return Container();
                return Text(snapshot.data, style: _textStyle);
            }
        ),

7 - Substitua `SECOND TODO` por um botão que realiza o envio de dados para
 uma rota do servidor
 
    RaisedButton(
        child: Text("I'm waiting", style: _textStyle,),
        onPressed: (){
             Stream2App.instance
                .create(route: 'product/customerSaid', body: 'I\'m waiting')
                .then((res) => print(res.isSuccess ? 'Success' : res.error.code));
        },
    )

Projeto pronto! Agora é só executar :)

Pelos links, você também pode visualizar esse projeto completo 
da seção "Começando" do [cliente em Flutter](example/getting_started)
e do [servidor em Node.js](https://github.com/DesiredApp/stream2app_nodejs_server/blob/master/example/getting_started).


## Issues

Sinta-se livre para abrir uma issue sobre:

- :grey_question: dúvidas

- :bulb: sugestões

- :page_facing_up: melhorias na documentação

- :ant: potenciais bugs


As issues devem ser escritas de preferência em inglês,
assim, todos poderão entender :grin:

## Licença

[MIT](LICENSE)
