# Documentation

:checkered_flag: [Português (Portuguese)](portugues_documentacao.md)

Documentation of the Flutter Client. 
[Click here](https://github.com/DesiredApp/stream2app_nodejs_server) to access 
the server side in Node.js.


 ## Important links
 *  [Getting Started](../README_PORTUGUES.md): Regarding to the client in Flutter.
 *  [Getting Started (server)](https://github.com/DesiredApp/stream2app_nodejs_server): Regarding to the server in Node.js.
 *  [chat (example)](../example/chat): Chat between the colors blue and green.
 *  [catalog (example)](../example/catalog): Users adding and removing products from a catalog.

## `init` - Configuring the client

The client can be initialized with the method `init`.

It's recommended to call `init` in the `main` method of the application.

### Params

#### serverUrl

The URL of the server, starts with `ws://` or `wss://`. Example: `ws://192.168.2.3:3123`.

#### projectName
 Name for this project (optional). 
 If `!= null`: the field `projectName` on server side must have the same name.

#### logger

 Allow customize the behavior of internal logs and enable/disable the default logger (optional).

#####  Params:
  
###### useDefaultLogger
 If `true`: the default logger will be used (optional). 
 Default: true.

###### useCustomLogger

 Allows the implementation of a custom logger (optional).

##### Example

    Stream2App.instance.init(
        projectName: 'MyApp',
        serverUrl: "ws://192.168.2.1:3123",
        logger: Logger(
            useDefaultLogger: false,
            useCustomLogger: (String message, Level level, {additionalData}) {
                final prefix = "> stream2app ["+level.toString().toUpperCase().substring(6)+"]: ";
                print(prefix+message);
                if(additionalData!=null)
                  print(additionalData.toString());
           }
        )
    );

## `connect` - Connecting to the server

Try perform a connection with the server.

In the server side, you can implement [grantConnection](https://github.com/DesiredApp/stream2app_nodejs_server/blob/master/documentation/english_documentation.md#grantconnection)
to accept or deny connections attempts from the client.

Returns the result of the connection attempt.

### Params

#### `ownClientId`
The ID of the user defined in your application.
This field must NOT be `null` when the user is logging in, 
otherwise must be `null` (optional).

#### `headers`
Allows informing the token of the respective `ownClientId` (and/or additional data)
so that the server can be able to accept or recuse the connection attempt (optional).

### Example

    Stream2App.instance.connect(ownClientId: 1, headers: {
         'Authorization' : 'TOKEN HERE'
    });

## `init` e `connect`
Where must you call `init` and `connect`? 

`init` must be called **only once**, preferably where the App starts,
therefore, is recommended that the initialization occur on `main.dart`.

`connect` can be called multiple times, 
since the user can do login and logout.


| Where                                                                                           |     `connect`      |       `init`       |
| ----------------------------------------------------------------------------------------------: |:------------------:|:------------------:|
| main.dart                                                                                       | :heavy_check_mark: | :heavy_check_mark: |
| When the user do login                                                                          | :heavy_check_mark: | :x:                |
| After a disconnect (example: user did logout) *                                                 | :heavy_check_mark: | :x:                |
| override `build` of a widget                                                                    | :x:                | :x:                |
| override `init` of a shared widget                                                             | :x:                | :x:                |
| override `init` of a widget that shows **one time**, when the App starts                       | :heavy_check_mark: | :heavy_check_mark: |

\* After a logout, it may be necessary for the user to read data from the server,
therefore, even after a logout can be done a `connect` with  
`ownClientId` being `null`.

## `reconnect()` - Reconnecting
Reconnects to the server using the same credentials
as the previous informed in `connect`.

Returns the result of the connection attempt.

## `disconnect()` - Disconnecting from the server
Stop the connection with the server and clear the credentials `headers` and `ownClientId`.

## `connection`
Get the status of the connection with the server.

## `disconnectReason`
May indicate the reason of no connection.

## `addOnConnectionChange`
Adds a `callback` that will be triggered 
every time the status of connection with 
the server changes.

`runCallbackNow` Default: true. If `true`: the `callback` is called
right after being added (optional).

## `removeOnConnectionChange`
Removes the added `callback `.

## `create`

 Creates data in the server.

#### Parâmetros

  `body` The data that will be created.

  `route` The path of the route.

  `query` Additional data (optional).

  `neverTimeout` Default: `false` (optional). If `true`: the
  request will be performed as soon as possible,
   without timeout.
  If `false`: the field `requestTimeoutInSeconds` defined in the server side
  will be the timeout.

#### Example
 
    Stream2App.instance
      .create(route: 'product',
        body: {
           'name' : 'Video Game',
           'price' : 500,
           'discount' : 0.1
        }
      ).then((res) => print(res.isSuccess ? 'Success' : res.error.code));
      

## `read`
 Read data once.

#### Parâmetros

 `route` The path of the route.

 `query` Additional data (optional), 
 here can be added a filter to indicate to the server
 which data this client will receive.

 `neverTimeout` Default: `false` (optional). If `true`: the
 request will be performed as soon as possible,
 without timeout.
 If `false`: the field `requestTimeoutInSeconds` defined in the server side
 will be the timeout.

#### Example
 
    Stream2App.instance
        .read(route: 'allProducts',
            query: {
                'nameContains' : 'game'
            },
            neverTimeout: true
        ).then((res) {
            (res.output as List).forEach((product) {
                print(product['name']);
            });
        });
      

## `listen`

 Get realtime data using `stream`.
 
 Returns a [Listening](#listening).

 Is necessary to call the method `Listening.close`
 for the server to stop sending data
 Example: @override `dispose` of Scaffold that uses this stream.

### Parâmetros

 `route` The path of the route.

 `query` Additional data (optional), 
 here can be added a filter to indicate to the server
 which data this client will receive.

### Example

    Listening listeningForNewGamingProducts;
    
    @override
    void initState() {
      listeningForNewGamingProducts = Stream2App.instance
          .listen(route: 'allProducts',
            query: {
              'nameContains' : 'game'
            },
          );
      listeningForNewGamingProducts.stream.listen((newRealtimeData) {
        List products = newRealtimeData.output;
        products.forEach((singleProduct) {
          print("New gaming product created: "+singleProduct['name']);
        });
      });
      super.initState();
    }
      
    @override
    void dispose() {
      listeningForNewGamingProducts.close(); //<--- don't forget to close the stream :)
      super.dispose();
    }

## `update`
 Updates data in the server.

#### Parâmetros

 `body` The entire data or field(s) that will be updated.

 `route` The path route.

 `query` Additional data (optional).

  `neverTimeout` Default: `false` (optional). If `true`: the
  request will be performed as soon as possible,
   without timeout.
  If `false`: the field `requestTimeoutInSeconds` defined in the server side
  will be the timeout.

#### Example

    Stream2App.instance
        .update(
            route: 'allProducts',
            query: {
              'nameContains' : 'game'
            },
            body: {
              'discount' : 0.8
            }
        ).then((res) => print(res.isSuccess ? 'Success' : res.error.code));

## `readAndBuild`
 Get data once and returns a 
 [FutureBuilder](https://api.flutter.dev/flutter/widgets/FutureBuilder-class.html)

#### Parâmetros

 `route` The path route.

 `builder` [Official documentation](https://api.flutter.dev/flutter/widgets/FutureBuilder/builder.html)

 `query` Additional data (optional), 
 here can be added a filter to indicate to the server
 which data this client will receive.

 `initialData` [Official documentation](https://api.flutter.dev/flutter/widgets/StreamBuilder/initialData.html) (optional).

 `key` [Official documentation](https://api.flutter.dev/flutter/foundation/Key-class.html) (optional).

## Example

    //other widgets...
    Stream2App.instance
        .readAndBuild(
          route: 'product',
          query: {
            'id': 1
          },
          builder: (context,  snapshot) {
            if(!snapshot.hasData)
              return Container();
            return Text(snapshot.data['name']);
          }
        );
    //other widgets...    

## `listenAndBuild`
 Get realtime data through [StreamBuilder](https://api.flutter.dev/flutter/widgets/StreamBuilder-class.html).

 
 Unlike the `listen` method, in `listenAndBuild`
 the stream will be closed automatically
 when this widget `dispose`.

### Parâmetros

 `route` The path route.

 `query` Additional data (optional), 
 here can be added a filter to indicate to the server
 which data this client will receive.

 `builder` [Official documentation](https://api.flutter.dev/flutter/widgets/StreamBuilder/builder.html)

 `initialData` [Official documentation](https://api.flutter.dev/flutter/widgets/StreamBuilder/initialData.html) (optional).

 `key` [Official documentation](https://api.flutter.dev/flutter/foundation/Key-class.html) (optional).

#### Example
   
    //other widgets...
    Stream2App.instance
        .listenAndBuild(
          route: 'allProducts',
          builder: (context,  snapshot) {
              if(!snapshot.hasData)
                return Container();
    
              final listOfProductsNames =
                  (snapshot.data as List)
                  .map((product) => Text(product['name'])).toList();
    
              return Column(
                children: listOfProductsNames,
              );
            }
          );
    //other widgets...
  
## `delete`
 Removes data from the server.

### Parâmetros

 `route` The path route.

 `query` Additional data, indicate here which data will be removed.

 `neverTimeout` Default: `false` (optional). If `true`: the
 request will be performed as soon as possible,
 without timeout.
 If `false`: the field `requestTimeoutInSeconds` defined in the server side
 will be the timeout.

#### Example

    Stream2App.instance
        .delete(
            route: 'product',
            query: {
              'id': 1
            },
        ).then((res) => print(res.isSuccess ? 'Success' : res.error.code));



